@@ -1,10 +1,9 @@
 import unittest
+from urllib.parse import unquote
 import jmespath
 from time import sleep
 from jsonschema import validate
 from jsonschema.exceptions import ValidationError
-from selenium.webdriver import Chrome
-from selenium.webdriver.remote.webdriver import WebDriver as SeleniumWebDriver
 from seldom.webdriver import WebDriver
 from seldom.request import HttpRequest
 from seldom.running.config import Seldom
@@ -49,6 +48,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         pass
 
     def setUp(self):
+        self.images = []
         self.start()
 
     def tearDown(self):
@@ -57,10 +57,8 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
     @property
     def driver(self):
         """
-        browser driver
+        return browser driver
         """
-        if isinstance(Seldom.driver, SeleniumWebDriver) is False:
-            Seldom.driver = Chrome()
         return Seldom.driver
 
     def assertTitle(self, title=None, msg=None):
@@ -72,7 +70,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         """
         if title is None:
             raise AssertionError("The assertion title cannot be empty.")
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
             try:
                 self.assertEqual(title, Seldom.driver.title)
                 log.info("👀 assert title: {title}.".format(
@@ -81,6 +79,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
             except AssertionError:
                 sleep(1)
         else:
+            print("abc")
             log.warn("❌ assert fail: {title}.".format(title=title))
             self.assertEqual(title, Seldom.driver.title, msg=msg)
 
@@ -93,7 +92,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         """
         if title is None:
             raise AssertionError("The assertion title cannot be empty.")
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
             try:
                 self.assertIn(title, Seldom.driver.title)
                 log.info("👀 assertIn title: {title}.".format(
@@ -114,10 +113,11 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         """
         if url is None:
             raise AssertionError("The assertion URL cannot be empty.")
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
+            current_url = unquote(Seldom.driver.current_url)
             try:
-                self.assertEqual(url, Seldom.driver.current_url)
-                log.info("👀 assert url: {url}.".format(url=Seldom.driver.current_url))
+                self.assertEqual(url, current_url)
+                log.info("👀 assert url: {url}.".format(url=current_url))
                 break
             except AssertionError:
                 sleep(1)
@@ -134,10 +134,11 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         """
         if url is None:
             raise AssertionError("The assertion URL cannot be empty.")
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
+            current_url = unquote(Seldom.driver.current_url)
             try:
-                self.assertIn(url, Seldom.driver.current_url)
-                log.info("👀 assertIn url: {url}.".format(url=Seldom.driver.current_url))
+                self.assertIn(url, current_url)
+                log.info("👀 assertIn url: {url}.".format(url=current_url))
                 break
             except AssertionError:
                 sleep(1)
@@ -156,7 +157,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
             raise AssertionError("The assertion text cannot be empty.")
 
         elem = Seldom.driver.find_element_by_tag_name("html")
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
             if elem.is_displayed():
                 try:
                     self.assertIn(text, elem.text)
@@ -188,7 +189,7 @@ class TestCase(unittest.TestCase, WebDriver, HttpRequest):
         """
         if msg is None:
             msg = "No elements found"
-        for _ in range(Seldom.timeout):
+        for _ in range(Seldom.timeout + 1):
             try:
                 log.info("👀 assertElement.")
                 self.get_elements(**kwargs)
