@@ -28,7 +28,7 @@ LOCATOR_LIST = {
 
 class WebElement(object):
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         if not kwargs:
             raise ValueError("Please specify a locator")
         if len(kwargs) > 1:
@@ -40,8 +40,9 @@ class WebElement(object):
         except KeyError:
             raise ValueError("Element positioning of type '{}' is not supported. ".format(self.by))
         self.find_elem_info = None
+        self.find_elem_warn = None
 
-    def __find_element(self, elem):
+    def __find_element(self, elem: tuple) -> None:
         """
         Find if the element exists.
         """
@@ -54,11 +55,10 @@ class WebElement(object):
             else:
                 time.sleep(1)
         else:
-            error_msg = "❌ Find 0 element through: {by}={value}".format(
+            self.find_elem_warn = "❌ Find 0 element through: {by}={value}".format(
                 by=elem[0], value=elem[1])
-            raise NotFindElementError(error_msg)
 
-    def get_elements(self):
+    def get_elements(self, index: int = None):
         """
         Judge element positioning way, and returns the element.
         """
@@ -90,8 +90,12 @@ class WebElement(object):
         else:
             raise NameError(
                 "Please enter the correct targeting elements,'id_/name/class_name/tag/link_text/xpath/css'.")
-
-        return elem
+        if index is None:
+            return elem
+        elif len(elem) == 0:
+            raise NotFindElementError(self.find_elem_warn)
+        else:
+            return elem[index]
 
     @staticmethod
     def show_element(elem):
@@ -125,6 +129,10 @@ class WebElement(object):
     def info(self):
         return self.find_elem_info
 
+    @property
+    def warn(self):
+        return self.find_elem_warn
+
 
 class WebDriver(object):
     """
@@ -141,65 +149,65 @@ class WebDriver(object):
             self.Keys(id_="kw").enter()
         """
 
-        def __init__(self, index=0, **kwargs):
+        def __init__(self, index: int  = 0, **kwargs) -> None:
             self.web_elem = WebElement(**kwargs)
-            self.elem = self.web_elem.get_elements()[index]
-            self.web_elem.show_element(self.elem)
+            self.elem = self.web_elem.get_elements()
+            self.web_elem.show_element(self.elem[index])
 
-        def input(self, text=""):
+        def input(self, text="") -> None:
             log.info("✅ {info}, input '{text}'.".format(info=self.web_elem.info, text=text))
             self.elem.send_keys(text)
 
-        def enter(self):
+        def enter(self) -> None:
             log.info("✅ {info}, enter.".format(info=self.web_elem.info))
             self.elem.send_keys(Keys.ENTER)
 
-        def select_all(self):
+        def select_all(self) -> None:
             log.info("✅ {info}, ctrl+a.".format(info=self.web_elem.info))
             if platform.system().lower() == "darwin":
                 self.elem.send_keys(Keys.COMMAND, "a")
             else:
                 self.elem.send_keys(Keys.CONTROL, "a")
 
-        def cut(self):
+        def cut(self) -> None:
             log.info("✅ {info}, ctrl+x.".format(info=self.web_elem.info))
             if platform.system().lower() == "darwin":
                 self.elem.send_keys(Keys.COMMAND, "x")
             else:
                 self.elem.send_keys(Keys.CONTROL, "x")
 
-        def copy(self):
+        def copy(self) -> None:
             log.info("✅ {info}, ctrl+c.".format(info=self.web_elem.info))
             if platform.system().lower() == "darwin":
                 self.elem.send_keys(Keys.COMMAND, "c")
             else:
                 self.elem.send_keys(Keys.CONTROL, "c")
 
-        def paste(self):
+        def paste(self) -> None:
             log.info("✅ {info}, ctrl+v.".format(info=self.web_elem.info))
             if platform.system().lower() == "darwin":
                 self.elem.send_keys(Keys.COMMAND, "v")
             else:
                 self.elem.send_keys(Keys.CONTROL, "v")
 
-        def backspace(self):
+        def backspace(self) -> None:
             log.info("✅ {info}, backspace.".format(info=self.web_elem.info))
             self.elem.send_keys(Keys.BACKSPACE)
 
-        def delete(self):
+        def delete(self) -> None:
             log.info("✅ {info}, delete.".format(info=self.web_elem.info))
             self.elem.send_keys(Keys.DELETE)
 
-        def tab(self):
+        def tab(self) -> None:
             log.info("✅ {info}, tab.".format(info=self.web_elem.info))
             self.elem.send_keys(Keys.TAB)
 
-        def space(self):
+        def space(self) -> None:
             log.info("✅ {info}, space.".format(info=self.web_elem.info))
             self.elem.send_keys(Keys.SPACE)
 
     @staticmethod
-    def visit(url):
+    def visit(url: str) -> None:
         """
         visit url.
 
@@ -211,7 +219,7 @@ class WebDriver(object):
             Seldom.driver = Chrome()
         Seldom.driver.get(url)
 
-    def open(self, url):
+    def open(self, url: str) -> None:
         """
         open url.
 
@@ -221,7 +229,7 @@ class WebDriver(object):
         self.visit(url)
 
     @staticmethod
-    def max_window():
+    def max_window() -> None:
         """
         Set browser window maximized.
 
@@ -231,7 +239,7 @@ class WebDriver(object):
         Seldom.driver.maximize_window()
 
     @staticmethod
-    def set_window(wide, high):
+    def set_window(wide: int = 0, high: int = 0) -> None:
         """
         Set browser window wide and high.
 
@@ -240,7 +248,7 @@ class WebDriver(object):
         """
         Seldom.driver.set_window_size(wide, high)
 
-    def type(self, text, clear=False, enter=False, index=0, **kwargs):
+    def type(self, text: str, clear: bool = False, enter: bool = False, index: int = 0, **kwargs) -> None:
         """
         Operation input box.
 
@@ -250,14 +258,14 @@ class WebDriver(object):
         if clear is True:
             self.clear(index, **kwargs)
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {info}, input '{text}'.".format(info=web_elem.info, text=text))
         elem.send_keys(text)
         if enter is True:
             elem.send_keys(Keys.ENTER)
 
-    def type_enter(self, text, clear=False, index=0, **kwargs):
+    def type_enter(self, text: str, clear: bool = False, index: int = 0, **kwargs) -> None:
         """
         Enter text and enter directly.
 
@@ -267,14 +275,14 @@ class WebDriver(object):
         if clear is True:
             self.clear(index, **kwargs)
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {info}, input '{text}' and enter.".format(info=web_elem.info, text=text))
         elem.send_keys(text)
         elem.send_keys(Keys.ENTER)
 
     @staticmethod
-    def clear(index=0, **kwargs):
+    def clear(index: int = 0, **kwargs) -> None:
         """
         Clear the contents of the input box.
 
@@ -282,13 +290,13 @@ class WebDriver(object):
             self.clear(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, clear input.".format(web_elem.info))
         elem.clear()
 
     @staticmethod
-    def click(index=0, **kwargs):
+    def click(index: int = 0, **kwargs) -> None:
         """
         It can click any text / image can be clicked
         Connection, check box, radio buttons, and even drop-down box etc..
@@ -297,13 +305,13 @@ class WebDriver(object):
             self.click(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, click.".format(web_elem.info))
         elem.click()
 
     @staticmethod
-    def slow_click(index=0, **kwargs):
+    def slow_click(index: int = 0, **kwargs) -> None:
         """
         Moving the mouse to the middle of an element. and click element.
 
@@ -311,13 +319,13 @@ class WebDriver(object):
             self.slow_click(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, slow click.".format(web_elem.info))
         ActionChains(Seldom.driver).move_to_element(elem).click(elem).perform()
 
     @staticmethod
-    def right_click(index=0, **kwargs):
+    def right_click(index: int = 0, **kwargs) -> None:
         """
         Right click element.
 
@@ -325,13 +333,13 @@ class WebDriver(object):
             self.right_click(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, right click.".find(web_elem.info))
         ActionChains(Seldom.driver).context_click(elem).perform()
 
     @staticmethod
-    def move_to_element(index=0, **kwargs):
+    def move_to_element(index: int = 0, **kwargs) -> None:
         """
         Mouse over the element.
 
@@ -339,13 +347,13 @@ class WebDriver(object):
             self.move_to_element(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, move to element.".format(web_elem.info))
         ActionChains(Seldom.driver).move_to_element(elem).perform()
 
     @staticmethod
-    def click_and_hold(index=0, **kwargs):
+    def click_and_hold(index: int = 0, **kwargs) -> None:
         """
         Mouse over the element.
 
@@ -353,13 +361,13 @@ class WebDriver(object):
             self.move_to_element(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, click and hold.".format(web_elem.info))
         ActionChains(Seldom.driver).click_and_hold(elem).perform()
 
     @staticmethod
-    def drag_and_drop_by_offset(index=0, x=0, y=0, **kwargs):
+    def drag_and_drop_by_offset(index:int = 0, x: int = 0, y: int = 0, **kwargs) -> None:
         """
         Holds down the left mouse button on the source element,
            then moves to the target offset and releases the mouse button.
@@ -370,14 +378,14 @@ class WebDriver(object):
          - y: Y offset to move to.
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         action = ActionChains(Seldom.driver)
         log.info("✅ {}, drag and drop by offset.".format(web_elem.info))
         action.drag_and_drop_by_offset(elem, x, y).perform()
 
     @staticmethod
-    def double_click(index=0, **kwargs):
+    def double_click(index: int = 0, **kwargs) -> None:
         """
         Double click element.
 
@@ -385,13 +393,13 @@ class WebDriver(object):
             self.double_click(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, double click.".format(web_elem.info))
         ActionChains(Seldom.driver).double_click(elem).perform()
 
     @staticmethod
-    def click_text(text, index=0):
+    def click_text(text: str, index: int = 0) -> None:
         """
         Click the element by the link text
 
@@ -399,13 +407,13 @@ class WebDriver(object):
             self.click_text("新闻")
         """
         web_elem = WebElement(link_text=text)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, click link.".format(web_elem.info))
         elem.click()
 
     @staticmethod
-    def close():
+    def close() -> None:
         """
         Closes the current window.
 
@@ -415,7 +423,7 @@ class WebDriver(object):
         Seldom.driver.close()
 
     @staticmethod
-    def quit():
+    def quit() -> None:
         """
         Quit the driver and close all the windows.
 
@@ -425,7 +433,7 @@ class WebDriver(object):
         Seldom.driver.quit()
 
     @staticmethod
-    def submit(index=0, **kwargs):
+    def submit(index: int = 0, **kwargs) -> None:
         """
         Submit the specified form.
 
@@ -433,13 +441,13 @@ class WebDriver(object):
             driver.submit(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, submit.".format(web_elem.info))
         elem.submit()
 
     @staticmethod
-    def refresh():
+    def refresh() -> None:
         """
         Refresh the current page.
 
@@ -450,7 +458,7 @@ class WebDriver(object):
         Seldom.driver.refresh()
 
     @staticmethod
-    def execute_script(script, *args):
+    def execute_script(script: str, *args):
         """
         Execute JavaScript scripts.
 
@@ -459,38 +467,30 @@ class WebDriver(object):
         """
         return Seldom.driver.execute_script(script, *args)
 
-    def window_scroll(self, width=None, height=None):
+    def window_scroll(self, width: int = 0, height: int = 0) -> None:
         """
         Setting width and height of window scroll bar.
 
         Usage:
             self.window_scroll(width=300, height=500)
         """
-        if width is None:
-            width = "0"
-        if height is None:
-            height = "0"
         js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
         self.execute_script(js)
 
-    def element_scroll(self, css, width=None, height=None):
+    def element_scroll(self, css: str, width: int = 0, height: int = 0) -> None:
         """
         Setting width and height of element scroll bar.
 
         Usage:
             self.element_scroll(css=".class", width=300, height=500)
         """
-        if width is None:
-            width = "0"
-        if height is None:
-            height = "0"
-        scroll_life = 'document.querySelector("{css}").scrollLeft = {w};'.format(css=css, w=width)
-        scroll_top = 'document.querySelector("{css}").scrollTop = {h};'.format(css=css, h=height)
+        scroll_life = 'document.querySelector("{css}").scrollLeft = {w};'.format(css=css, w=str(width))
+        scroll_top = 'document.querySelector("{css}").scrollTop = {h};'.format(css=css, h=str(height))
         self.execute_script(scroll_life)
         self.execute_script(scroll_top)
 
     @staticmethod
-    def get_attribute(attribute=None, index=0, **kwargs):
+    def get_attribute(attribute=None, index: int = 0, **kwargs) -> str:
         """
         Gets the value of an element attribute.
 
@@ -500,13 +500,13 @@ class WebDriver(object):
         if attribute is None:
             raise ValueError("attribute is not None")
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {info}, get attribute：{att}.".format(info=web_elem.info, att=attribute))
         return elem.get_attribute(attribute)
 
     @staticmethod
-    def get_text(index=0, **kwargs):
+    def get_text(index: int = 0, **kwargs) -> str:
         """
         Get element text information.
 
@@ -514,13 +514,13 @@ class WebDriver(object):
             self.get_text(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {info}, get text: {text}.".format(info=web_elem.info, text=elem.text))
         return elem.text
 
     @staticmethod
-    def get_display(index=0, **kwargs):
+    def get_display(index: int = 0, **kwargs) -> bool:
         """
         Gets the element to display,The return result is true or false.
 
@@ -528,14 +528,14 @@ class WebDriver(object):
             self.get_display(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         result = elem.is_displayed()
         log.info("✅ {info}, element is display: {r}.".format(info=web_elem.info, r=result))
         return result
 
     @property
-    def get_title(self):
+    def get_title(self) -> str:
         """
         Get window title.
 
@@ -546,7 +546,7 @@ class WebDriver(object):
         return Seldom.driver.title
 
     @property
-    def get_url(self):
+    def get_url(self) -> str:
         """
         Get the URL address of the current page.
 
@@ -557,7 +557,7 @@ class WebDriver(object):
         return Seldom.driver.current_url
 
     @property
-    def get_alert_text(self):
+    def get_alert_text(self) -> str:
         """
         Gets the text of the Alert.
 
@@ -568,7 +568,7 @@ class WebDriver(object):
         return Seldom.driver.switch_to.alert.text
 
     @staticmethod
-    def wait(secs=10):
+    def wait(secs: int = 10) -> None:
         """
         Implicitly wait.All elements on the page.
 
@@ -579,7 +579,7 @@ class WebDriver(object):
         Seldom.driver.implicitly_wait(secs)
 
     @staticmethod
-    def accept_alert():
+    def accept_alert() -> None:
         """
         Accept warning box.
 
@@ -590,7 +590,7 @@ class WebDriver(object):
         Seldom.driver.switch_to.alert.accept()
 
     @staticmethod
-    def dismiss_alert():
+    def dismiss_alert() -> None:
         """
         Dismisses the alert available.
 
@@ -601,7 +601,7 @@ class WebDriver(object):
         Seldom.driver.switch_to.alert.dismiss()
 
     @staticmethod
-    def switch_to_frame(index=0, **kwargs):
+    def switch_to_frame(index: int = 0, **kwargs) -> None:
         """
         Switch to the specified frame.
 
@@ -609,13 +609,13 @@ class WebDriver(object):
             self.switch_to_frame(css="#el")
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, switch to frame.".format(web_elem.info))
         Seldom.driver.switch_to.frame(elem)
 
     @staticmethod
-    def switch_to_frame_out():
+    def switch_to_frame_out() -> None:
         """
         Returns the current form machine form at the next higher level.
         Corresponding relationship with switch_to_frame () method.
@@ -627,7 +627,7 @@ class WebDriver(object):
         Seldom.driver.switch_to.default_content()
 
     @staticmethod
-    def switch_to_window(window):
+    def switch_to_window(window: int) -> None:
         """
         Switches focus to the specified window.
 
@@ -641,8 +641,7 @@ class WebDriver(object):
         all_handles = Seldom.driver.window_handles
         Seldom.driver.switch_to.window(all_handles[window])
 
-    @staticmethod
-    def screenshots(file_path):
+    def screenshots(self, file_path: str = None) -> None:
         """
         Saves a screenshots of the current window to a PNG image file.
 
@@ -650,10 +649,13 @@ class WebDriver(object):
         self.screenshots('/Screenshots/foo.png')
         """
         log.info("📷️  screenshot.")
-        Seldom.driver.save_screenshot(file_path)
+        if file_path is None:
+            self.images.append(Seldom.driver.get_screenshot_as_base64())
+        else:
+            Seldom.driver.save_screenshot(file_path)
 
     @staticmethod
-    def select(value=None, text=None, index=None, **kwargs):
+    def select(value: str = None, text: str = None, index: int = None, **kwargs) -> None:
         """
         Constructor. A check is made that the given element is, indeed, a SELECT tag. If it is not,
         then an UnexpectedTagNameException is thrown.
@@ -674,7 +676,7 @@ class WebDriver(object):
             self.select(css="#nr", index=2)
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         web_elem.show_element(elem)
         log.info("✅ {}, select option.".format(web_elem.info))
         if value is not None:
@@ -688,7 +690,7 @@ class WebDriver(object):
                 '"value" or "text" or "index" options can not be all empty.')
 
     @staticmethod
-    def get_cookies():
+    def get_cookies() -> list:
         """
         Returns a set of dictionaries, corresponding to cookies visible in the current session.
         Usage:
@@ -697,16 +699,16 @@ class WebDriver(object):
         return Seldom.driver.get_cookies()
 
     @staticmethod
-    def get_cookie(name):
+    def get_cookie(name: str) -> dict:
         """
         Returns information of cookie with ``name`` as an object.
         Usage:
-            self.get_cookie()
+            self.get_cookie("name")
         """
         return Seldom.driver.get_cookie(name)
 
     @staticmethod
-    def add_cookie(cookie_dict):
+    def add_cookie(cookie_dict: dict) -> None:
         """
         Adds a cookie to your current session.
         Usage:
@@ -718,7 +720,7 @@ class WebDriver(object):
             raise TypeError("Wrong cookie type.")
 
     @staticmethod
-    def add_cookies(cookie_list):
+    def add_cookies(cookie_list: list) -> None:
         """
         Adds a cookie to your current session.
         Usage:
@@ -738,7 +740,7 @@ class WebDriver(object):
             raise TypeError("Wrong cookie type.")
 
     @staticmethod
-    def delete_cookie(name):
+    def delete_cookie(name: str) -> None:
         """
         Deletes a single cookie with the given name.
         Usage:
@@ -747,7 +749,7 @@ class WebDriver(object):
         Seldom.driver.delete_cookie(name)
 
     @staticmethod
-    def delete_all_cookies():
+    def delete_all_cookies() -> None:
         """
         Delete all cookies in the scope of the session.
         Usage:
@@ -756,7 +758,7 @@ class WebDriver(object):
         Seldom.driver.delete_all_cookies()
 
     @staticmethod
-    def sleep(sec):
+    def sleep(sec: int) -> None:
         """
         Usage:
             self.sleep(seconds)
@@ -765,7 +767,7 @@ class WebDriver(object):
         time.sleep(sec)
 
     @staticmethod
-    def check_element(css=None):
+    def check_element(css: str = None) -> None:
         """
         Check that the element exists
 
@@ -796,12 +798,15 @@ class WebDriver(object):
         print(len(ret))
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()
-        log.info("✅ {}.".format(web_elem.info))
-        return elem
+        elems = web_elem.get_elements()
+        if len(elems) == 0:
+            log.warn("{}.".format(web_elem.warn))
+        else:
+            log.info("✅ {}.".format(web_elem.info))
+        return elems
 
     @staticmethod
-    def get_element(index=0, **kwargs):
+    def get_element(index: int=0, **kwargs):
         """
         Get a set of elements
 
@@ -810,6 +815,6 @@ class WebDriver(object):
         elem.click()
         """
         web_elem = WebElement(**kwargs)
-        elem = web_elem.get_elements()[index]
+        elem = web_elem.get_elements(index)
         log.info("✅ {}.".format(web_elem.info))
         return elem
